"""
Example using a dictionary as specification instead of YAML
"""
from flask import Flask, jsonify

from flasgger import Swagger
from flasgger.utils import swag_from

app = Flask(__name__)
app.config['SWAGGER'] = {
    'title': 'Colors API',
    'uiversion': 2
}
swag = Swagger(app)


colors_spec = {
  "tags": [
    "colors"
  ],
  "parameters": [
    {
      "name": "palette",
      "in": "path",
      "type": "string",
      "enum": [
        "all",
        "rgb",
        "cmyk"
      ],
      "required": True,
      "default": "all",
      "description": "Which palette to filter?"
    }
  ],
  "operationId": "get_colors",
  "consumes": [
    "application/json"
  ],
  "produces": [
    "application/json"
  ],
  "security": {
    "colors_auth": [
      "write:colors",
      "read:colors"
    ]
  },
  "schemes": [
    "http",
    "https"
  ],
  "deprecated": False,
  "externalDocs": {
    "description": "Project repository",
    "url": "http://github.com/rochacbruno/flasgger"
  },
  "definitions": {
    "Palette": {
      "type": "object",
      "properties": {
        "palette_name": {
          "type": "array",
          "items": {
            "$ref": "#/definitions/Color"
          }
        }
      }
    },
    "Color": {
      "type": "string"
    }
  },
  "responses": {
    "200": {
      "description": "A list of colors (may be filtered by palette)",
      "schema": {
        "$ref": "#/definitions/Palette"
      },
      "examples": {
        "rgb": [
          "red",
          "green",
          "blue"
        ]
      }
    }
  }
}


@app.route('/colors/<palette>/')
@swag_from(colors_spec)
def colors(palette):
    """
    Example using a dictionary as specification
    This is the description
    You can also set 'summary' and 'description' in
    specs_dict
    ---
    # values here overrides the specs dict
    deprecated: true
    """
    all_colors = {
        'cmyk': ['cian', 'magenta', 'yellow', 'black'],
        'rgb': ['red', 'green', 'blue']
    }
    if palette == 'all':
        result = all_colors
    else:
        result = {palette: all_colors.get(palette)}

    return jsonify(result)

if __name__ == "__main__":
    app.run(debug=True)



"""
Ensure that top-level vendor extension config is preserved
"""

from flasgger import Swagger
from flask import Flask, jsonify

app = Flask(__name__)
app.config['SWAGGER'] = {
    'title': 'Vendor extension test',
    'uiversion': 2,
    'x-groupTag': 'Test',
}
swag = Swagger(app)


def test_swag(client, specs_data):
    """
    This test is runs automatically in Travis CI

    :param client: Flask app test client
    :param specs_data: {'url': {swag_specs}} for every spec in app
    """
    assert 'x-groupTag' in specs_data['/apispec_1.json']
    assert specs_data['/apispec_1.json']['x-groupTag'] == 'Test'


if __name__ == "__main__":
    app.run(debug=True)



"""
In this example Swagger UI is disabled.
"""
from flask import Flask
try:
    from http import HTTPStatus
except ImportError:
    import httplib as HTTPStatus
from flasgger import Swagger

swagger_config = {
    'headers': [
    ],
    'specs': [
        {
            'endpoint': 'apispec',
            'route': '/apispec.json',
            'rule_filter': lambda rule: True,
            'model_filter': lambda tag: True,
        }
    ],
    'swagger_ui': False
}

app = Flask(__name__)
swag = Swagger(app, config=swagger_config)


def test_swag(client, specs_data):
    """
    This test is runs automatically in Travis CI

    :param client: Flask app test client
    :param specs_data: {'url': {swag_specs}} for every spec in app
    """

    assert not specs_data
    assert client.get('/apidocs/').status_code == HTTPStatus.NOT_FOUND
    assert client.get('/apispec.json').status_code == HTTPStatus.OK

if __name__ == '__main__':
    app.run(debug=True)



"""
Test compatibility with pre 0.5.14 version

- produces was accepted as string
- definitions references had method prefix
"""


from flask import Flask, jsonify, request

from flasgger import Swagger

app = Flask(__name__)
app.config['SWAGGER'] = {
    # set to True so instead of
    # $ref: '#/definitions/alert'
    # we get
    # $ref: '#/definitions/index_post_alert'
    'prefix_ids': True
}
swag = Swagger(app)


@app.route('/', methods=['POST'])
def index():
    """
    Test the compatibility
    This is just a test for flasgger 0.5.14 compat
    ---
    parameters:
      - in: body
        name: body
        schema:
          id: alert
          properties:
            message:
              type: string
    produces:
      application/json
    responses:
      200:
        description: Alert
        schema:
          $ref: '#/definitions/index_post_alert'
    """
    return jsonify({'message': request.json.get('message')})


if __name__ == '__main__':
    app.run(debug=True)



"""
In this example `openapi` version is used instead of `swagger` version.
"""
from flask import Flask
from flasgger import Swagger

app = Flask(__name__)

swagger_config = {
    "headers": [],
    "openapi": "3.0.2",
    "components": {
        "securitySchemes": {
            "oAuthSample": {
                "type": "oauth2",
                "flows": {
                    "clientCredentials": {
                        "tokenUrl": "https://api.pgsmartshopassistant.com/o/token/",
                    }
                }
            }
        },
    },
    "servers": [
        {
            "url": "https://api.example.com/v1",
            "description": "Production server (uses live data)"
        },
        {
            "url": "https://sandbox-api.example.com:8443/v1",
            "description": "Sandbox server (uses test data)"
        }
    ],
    "specs": [
        {
            "endpoint": "swagger",
            "route": "/characteristics/swagger.json",
            "rule_filter": lambda rule: True,  # all in
            "model_filter": lambda tag: True,  # all in
        }
    ],
    "title": "Product Characteristics API",
    "version": '',
    "termsOfService": "",
    "static_url_path": "/characteristics/static",
    "swagger_ui": True,
    "specs_route": "/characteristics/swagger/",
    "description": "",
}

colors_spec = {
  "tags": [
    "colors"
  ],
  "parameters": [
    {
      "name": "palette",
      "in": "path",
      "type": "string",
      "enum": [
        "all",
        "rgb",
        "cmyk"
      ],
      "required": True,
      "default": "all",
      "description": "Which palette to filter?"
    }
  ],
  "operationId": "get_colors",
  "consumes": [
    "application/json"
  ],
  "produces": [
    "application/json"
  ],
  "security": {
    "colors_auth": [
      "write:colors",
      "read:colors"
    ]
  },
  "schemes": [
    "http",
    "https"
  ],
  "deprecated": False,
  "externalDocs": {
    "description": "Project repository",
    "url": "http://github.com/rochacbruno/flasgger"
  },
  "definitions": {
    "Palette": {
      "type": "object",
      "properties": {
        "palette_name": {
          "type": "array",
          "items": {
            "$ref": "#/definitions/Color"
          }
        }
      }
    },
    "Color": {
      "type": "string"
    }
  },
  "responses": {
    "200": {
      "description": "A list of colors (may be filtered by palette)",
      "schema": {
        "$ref": "#/definitions/Palette"
      },
      "examples": {
        "rgb": [
          "red",
          "green",
          "blue"
        ]
      }
    }
  }
}


swag = Swagger(app, config=swagger_config)


def test_swag(client, specs_data):
    """
    This test is runs automatically in Travis CI

    :param client: Flask app test client
    :param specs_data: {'url': {swag_specs}} for every spec in app
    """
    for spec in specs_data.values():
        assert 'openapi' in spec
        assert '3.0.2' == spec['openapi']
        assert 'swagger' not in spec
        assert 'components' in spec
        assert 'securitySchemes' in spec['components']
        assert 'oAuthSample' in spec['components']['securitySchemes']

        assert 'servers' in spec  # See issue #366


if __name__ == '__main__':
    app.run(debug=True)



import json
import random

from flasgger import Swagger
from flasgger.utils import get_examples


def get_specs_data(mod):
    """
    return all specs dictionary for some app
    """
    # for each example app in /examples folder
    client = mod.app.test_client()
    # init swag if not yet inititalized (no-routes example)
    specs_route = None
    specs_data = {}
    if getattr(mod.app, 'swag', None) is None:
        _swag = Swagger()
        _swag.config['endpoint'] = str(random.randint(1, 5000))
        _swag.init_app(mod.app)
    # get all the specs defined for the example app
    else:
        try:
            flasgger_config = mod.swag.config

            if flasgger_config.get('swagger_ui') is False:
                return specs_data

            specs_route = flasgger_config.get('specs_route', '/apidocs/')
        except AttributeError:
            pass

    if specs_route is None:
        specs_route = '/apidocs/'

    apidocs = client.get('?'.join((specs_route, 'json=true')))
    specs = json.loads(apidocs.data.decode("utf-8")).get('specs')

    for spec in specs:
        # for each spec get the spec url
        url = spec['url']
        response = client.get(url)
        decoded = response.data.decode("utf-8")
        specs_data[url] = json.loads(decoded)

    return specs_data


def get_test_metadata(mod):
    """Create a dictionary of test metadata defined in an example

    Every top-level constant prefixed with "_TEST_META_" is treated as
    metadata which may control test behavior. The prefix is stripped and the
    remaining text is lowercased to form the key in the metadata dictionary.

    Example: '_TEST_META_SKIP_FULL_VALIDATION' -> 'skip_full_validation'
    """

    test_metadata_prefix = '_TEST_META_'
    return {key[len(test_metadata_prefix):].lower(): getattr(mod, key)
            for key in mod.__dict__
            if key.startswith(test_metadata_prefix)}


def pytest_generate_tests(metafunc):
    """
    parametrize tests using examples() function
    to generate one test for each examples/
    """

    if 'test_data' in metafunc.fixturenames:
        test_data = [
            (mod, mod.app.test_client(),
             get_specs_data(mod), get_test_metadata(mod))
            for mod in get_examples()
        ]

        metafunc.parametrize(
            'test_data',
            test_data,
            ids=lambda x: x[0].__name__
        )



# -*- coding: utf-8 -*-

"""
Example to test cascading $refs and basePath
"""

from flask import Flask, jsonify, Blueprint
from flasgger import swag_from, Swagger
try:
    import simplejson as json
except ImportError:
    import json

try:
    from http import HTTPStatus
except ImportError:
    import httplib as HTTPStatus


swagger_description = 'test_refs_basePath_API.yaml'

app = Flask(__name__)

swagger = Swagger(app, template_file=swagger_description)

api_blueprint = Blueprint('api', 'api', url_prefix='/api')


@api_blueprint.route('/get_cost', methods=['POST'])
@swag_from(swagger.template, definition='GetCostRequest', validation=True)
def get_cost():
    result = dict(description='The best place',
                  cost=dict(currency='EUR', value=123456))
    return jsonify([result])


app.register_blueprint(api_blueprint)


def test_swag(client, specs_data):
    """
    This test is runs automatically in Travis CI

    :param client: Flask app test client
    :param specs_data: {'url': {swag_specs}} for every spec in app
    """

    with client.get('apispec_1.json') as response:
        assert response.status_code == HTTPStatus.OK

        response_data = json.loads(response.data)

        assert response_data.get('basePath') == '/api/', \
            'wrong basePath: %r' % response_data

        paths = response_data.get('paths')
        assert '/get_cost' in paths, \
            'get_cost NOK: %r' % response_data
        assert '/api/get_costs' not in paths, \
            '/api/get_cost NOK: %r' % response_data

    req_data = dict(level=2,
                    location=dict(name='my preferred location',
                                  position=dict(latitude=47.352735,
                                                longitude=0.593682)))

    with client.post('/api/get_cost', data=json.dumps(req_data),
                     content_type='application/json') as response:
        assert response.status_code == HTTPStatus.OK, \
            'bad status: %r' % response.data


if __name__ == '__main__':
    app.run(debug=True)



"""
The simple example using declared definitions, and external static js/css.
"""

from flask import Flask, jsonify

from flasgger import Swagger

app = Flask(__name__)
app.config['SWAGGER'] = {
    'title': 'Colors API'
}
swagger_config = Swagger.DEFAULT_CONFIG
swagger_config['swagger_ui_bundle_js'] = '//unpkg.com/swagger-ui-dist@3/swagger-ui-bundle.js'
swagger_config['swagger_ui_standalone_preset_js'] = '//unpkg.com/swagger-ui-dist@3/swagger-ui-standalone-preset.js'
swagger_config['jquery_js'] = '//unpkg.com/jquery@2.2.4/dist/jquery.min.js'
swagger_config['swagger_ui_css'] = '//unpkg.com/swagger-ui-dist@3/swagger-ui.css'
Swagger(app, config=swagger_config)

@app.route('/colors/<palette>/')
def colors(palette):
    """Example endpoint return a list of colors by palette
    This is using docstring for specifications
    ---
    tags:
      - colors
    parameters:
      - name: palette
        in: path
        type: string
        enum: ['all', 'rgb', 'cmyk']
        required: true
        default: all
        description: Which palette to filter?
    operationId: get_colors
    consumes:
      - application/json
    produces:
      - application/json
    security:
      colors_auth:
        - 'write:colors'
        - 'read:colors'
    schemes: ['http', 'https']
    deprecated: false
    externalDocs:
      description: Project repository
      url: http://github.com/rochacbruno/flasgger
    definitions:
      Palette:
        type: object
        properties:
          palette_name:
            type: array
            items:
              $ref: '#/definitions/Color'
      Color:
        type: string
    responses:
      200:
        description: A list of colors (may be filtered by palette)
        schema:
          $ref: '#/definitions/Palette'
        examples:
          rgb: ['red', 'green', 'blue']
    """
    all_colors = {
        'cmyk': ['cian', 'magenta', 'yellow', 'black'],
        'rgb': ['red', 'green', 'blue']
    }
    if palette == 'all':
        result = all_colors
    else:
        result = {palette: all_colors.get(palette)}

    return jsonify(result)


def test_swag(client, specs_data):
    """
    This test is runs automatically in Travis CI

    :param client: Flask app test client
    :param specs_data: {'url': {swag_specs}} for every spec in app
    """
    for url, spec in specs_data.items():
        assert 'Palette' in spec['definitions']
        assert 'Color' in spec['definitions']
        assert 'colors' in spec['paths']['/colors/{palette}/']['get']['tags']


if __name__ == "__main__":
    app.run(debug=True)



"""
In this example nothing is swagged
this is useful for testing if Swagger is ignoring
no swagabble views.
"""
from flask import Flask
from flask.views import MethodView

from flasgger import Swagger

app = Flask(__name__)
swag = Swagger()


@app.route('/')
def index():
    return 'Hello World'


class TestView(MethodView):
    def get(self):
        return 'Hello World'


class Meow(MethodView):
    """
    This is to ensure Swagger does not break with empty MethodViews
    issue #76
    """
    pass

app.add_url_rule(
    '/meow/<int:param>/',
    view_func=Meow.as_view('meow'),
    methods=['DELETE']
)


app.add_url_rule(
    '/testview',
    view_func=TestView.as_view('testview'),
    methods=['GET']
)


if __name__ == '__main__':
    swag.init_app(app)
    app.run(debug=True)



"""
This script is a WSGI configuration/setup file suitable for use in
deploying the Flasgger demo_app to PythonAnywhere, https://www.pythonanywhere.com/ .

To deploy:

- Create a PythonAnywhere account if you don't already have one. Replace <username>
  below with your PythonAnywhere username.
- Visit https://www.pythonanywhere.com/user/<username>/webapps and create a new/default
  web app, using Manual Configuration and Python 3.6.
- Confirm you can access the Hello, World! page at https://<username>.pythonanywhere.com/ .
- Visit https://www.pythonanywhere.com/user/<username>/consoles/ and create a new bash console.
- From your home directory /home/<username>, run:
      git clone https://github.com/rochacbruno/flasgger.git
  ... or use another fork.
- cd ~/flasgger, then change to whichever branch you want to use.
- Create a new virtual environment:
      mkvirtualenv --python=python3.6
- Run: pip3 install -r requirements.txt -r requirements-dev.txt
- cd /var/www
- rm ${USERNAME}_pythonanywhere_com_wsgi.py
- ln -s ~/flasgger/etc/pythonanywhere/pythonanywhere_com_wsgi.py ${USERNAME}_pythonanywhere_com_wsgi.py
- Go to the web app console at https://www.pythonanywhere.com/user/<username>/webapps .
- Modify the configuration:
  - Make the source path /home/<username>/flasgger
  - Make the working directory /home/<username>/flasgger
  - Change the virtual env path to /home/<username>/.virtualenvs/flasgger
- Reload the web app.
- Visit https://<username>.pythonanywhere.com/ and you should see the demo_app landing page.
- If an error occurs, review the log files.
"""
import os
import subprocess
import sys

home = os.path.expanduser("~")
path = os.environ.get('FLASGGER_HOME') or home + '/flasgger'

subprocess.check_call(['git', 'pull', path])

if path not in sys.path:
    sys.path.append(path)

from demo_app.app import application



"""
An example that uses HTTP Basic Auth to protect the /apidocs and
/apispec_1.json endpoints.

In this example the login credentials to view the Swagger docs are as follows:

    Username: guest
    Password: secret

Failing to provide the right credentials results in a 401 Unauthorized
response from all the Flasgger endpoints.
"""
from flask import Flask, request, redirect, Response, jsonify
from functools import wraps

from flasgger import Swagger

def requires_basic_auth(f):
    """Decorator to require HTTP Basic Auth for your endpoint."""

    def check_auth(username, password):
        return username == "guest" and password == "secret"

    def authenticate():
        return Response(
            "Authentication required.", 401,
            {"WWW-Authenticate": "Basic realm='Login Required'"},
        )

    @wraps(f)
    def decorated(*args, **kwargs):
        # NOTE: This example will require Basic Auth only when you run the
        # app directly. For unit tests, we can't block it from getting the
        # Swagger specs so we just allow it to go thru without auth.
        # The following two lines of code wouldn't be needed in a normal
        # production environment.
        if __name__ != "__main__":
            return f(*args, **kwargs)

        auth = request.authorization
        if not auth or not check_auth(auth.username, auth.password):
            return authenticate()
        return f(*args, **kwargs)

    return decorated

app = Flask(__name__)
app.config["SWAGGER"] = {
    "title": "Swagger Basic Auth App",
    "uiversion": 2,
}
swag = Swagger(app,
    decorators=[ requires_basic_auth ],
    template={
        "swagger": "2.0",
        "info": {
            "title": "Swagger Basic Auth App",
            "version": "1.0",
        },
        "consumes": [
            "application/json",
        ],
        "produces": [
            "application/json",
        ],
    },
)

@app.route("/echo/<name>", methods=["GET", "POST"])
def echo(name):
    """
    Echo back the name and any posted parameters.
    ---
    tags:
      - echo
    parameters:
      - in: path
        name: name
        type: string
        required: true
      - in: body
        name: body
        description: JSON parameters.
        schema:
          properties:
            first_name:
              type: string
              description: First name.
              example: Alice
            last_name:
              type: string
              description: Last name.
              example: Smith
            dob:
              type: string
              format: date
              description: Date of birth.
              example: 1990-01-01
            comment:
              type: string
              description: Something arbitrary.
              example: Hello world
    responses:
      200:
        description: OK.
    """
    data = {
        "url_name": name,
        "json": request.json,
    }
    return jsonify(data)

@app.route("/")
def index():
    return redirect("/apidocs")

if __name__ == "__main__":
    app.run(debug=True)



"""
A test to ensure routes from Blueprints are swagged as expected.
"""
from flask import Blueprint, Flask, jsonify

from flasgger import Swagger
from flasgger.utils import swag_from

app = Flask(__name__)

example_blueprint = Blueprint("example_blueprint", __name__)


@example_blueprint.route('/usernames/<username>', methods=['GET', 'POST'])
@swag_from('username_specs.yml', methods=['GET'])
@swag_from('username_specs.yml', methods=['POST'])
def usernames(username):
    return jsonify({'username': username})


@example_blueprint.route('/usernames2/<username>', methods=['GET', 'POST'])
def usernames2(username):
    """
    This is the summary defined in yaml file
    First line is the summary
    All following lines until the hyphens is added to description
    the format of the first lines until 3 hyphens will be not yaml compliant
    but everything below the 3 hyphens should be.
    ---
    tags:
      - users
    parameters:
      - in: path
        name: username
        type: string
        required: true
    responses:
      200:
        description: A single user item
        schema:
          id: rec_username
          properties:
            username:
              type: string
              description: The name of the user
              default: 'steve-harris'
    """
    return jsonify({'username': username})


@example_blueprint.route('/users', endpoint='user-without-id', methods=['GET'])
@example_blueprint.route('/users/<user_id>', endpoint='user-with-id', methods=['GET'])
@swag_from('user_with_id_specs.yml', endpoint='example_blueprint.user-with-id', methods=['GET'])
@swag_from('user_without_id_specs.yml', endpoint='example_blueprint.user-without-id', methods=['GET'])
def usernames(user_id=None):
    if user_id:
        return jsonify({'user_id': user_id})
    else:
        return jsonify([])


app.register_blueprint(example_blueprint)

swag = Swagger(app)

if __name__ == "__main__":
    app.run(debug=True)




#!/usr/bin/python
import argparse
from collections import OrderedDict

from flask import Flask, render_template
from werkzeug.serving import run_simple
try:
    from werkzeug.middleware.dispatcher import DispatcherMiddleware
except ImportError:
    from werkzeug.wsgi import DispatcherMiddleware

from flasgger import __version__
from flasgger.utils import get_examples

app = Flask(__name__)

examples = OrderedDict({
    '/{0}'.format(mod.__name__.split('.')[-1]): mod
    for mod in sorted(get_examples(), key=lambda x: x.__name__)
})


@app.route('/')
def index():
    return render_template('flasgger.html',
                           examples=examples,
                           version=__version__)


for example, mod in examples.items():
    if hasattr(mod.app, 'swag'):
        mod.app.swag.config['basePath'] = example

application = DispatcherMiddleware(
    app,
    {name: mod.app for name, mod in examples.items()}
)


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Run App for WSGI")
    parser.add_argument('-p', '--port', help='App Port')
    parser.add_argument('-i', '--host', help='App Host')
    parser.add_argument('-r', '--reloader', action='store_true',
                        help='Turn reloader on')
    parser.add_argument('-d', '--debug', action='store_true',
                        help='Turn debug on')
    args = parser.parse_args()

    run_simple(
        args.host or '0.0.0.0',
        int(args.port) if args.port else 5000,
        application,
        use_reloader=args.reloader or False,
        use_debugger=args.debug or False,
    )



try:
    from setuptools import setup, find_packages
except ImportError:
    from distutils.core import setup, find_packages


setup(
    name='flasgger_package',
    version='0.0.1',
    url='https://github.com/rochacbruno/flasgger/',
    license='MIT',
    author='testing@flasgger',
    author_email='testing@flasgger',
    description='A package to use during local and CI tests',
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    platforms='any',
    install_requires=[
        'flasgger>=0.9.4.dev0',
    ]
)



"""
In this example `openapi` version is used instead of `swagger` version.
"""
from flask import Flask, jsonify, request
from flasgger import Swagger, swag_from

app = Flask(__name__)
swag = Swagger(app, config={
    'headers': [],
    'specs': [
        {
            'endpoint': 'apispec',
            'route': '/apispec.json'
        }
    ],
    'openapi': '3.0.1'
})


@swag.definition('Pet')
class Pet(object):
    """
    Pet Object
    ---
    properties:
        name:
            type: string
    """
    def __init__(self, name):
        self.name = str(name)

    def dump(self):
        return dict(vars(self).items())


@app.route('/requestBody', methods=['POST'])
@swag_from()
def request_body_endpoint():
    """
    An endpoint for testing requestBody documentation.
    ---
    description: Post a request body
    requestBody:
        content:
            application/json:
                schema:
                    $ref: '#/definitions/Pet'
        required: true
    responses:
        200:
            description: The posted request body
            content:
                application/json:
                    schema:
                        $ref: '#/definitions/Pet'
    """
    return jsonify(request.json)


def test_swag(client, specs_data):
    """
    This test is runs automatically in Travis CI

    :param client: Flask app test client
    :param specs_data: {'url': {swag_specs}} for every spec in app
    """
    for url, spec in specs_data.items():
        assert 'Pet' in spec['definitions']

        assert 'paths' in spec
        paths = spec['paths']
        for path_def in paths.values():
            for method_def in path_def.values():
                assert 'requestBody' in method_def


if __name__ == '__main__':
    app.run(debug=True)



# coding: utf-8
"""
This tests the use of a view coming from installed
package.
"""

from flask import Flask, jsonify
from flasgger import Swagger
from flasgger_package import package_view


app = Flask(__name__)
swag = Swagger(app)

app.add_url_rule(
    '/v1/decorated/<username>',
    view_func=package_view
)


@app.route('/v2/decorated/<username>')
def package_view_2(username):
    """
    This is the summary defined in yaml file
    First line is the summary
    All following lines until the hyphens is added to description
    the format of the first lines until 3 hyphens will be not yaml compliant
    but everything below the 3 hyphens should be.
    ---
    tags:
      - users
    import: "flasgger_package/parameters.yml"
    responses:
      200:
        description: A single user item
        schema:
          id: rec_username
          properties:
            username:
              type: string
              description: The name of the user
              default: 'steve-harris 2'
    """
    return jsonify({'username': username})


def test_swag(client, specs_data):
    """
    This test is runs automatically in Travis CI

    :param client: Flask app test client
    :param specs_data: {'url': {swag_specs}} for every spec in app
    """
    for url, spec in specs_data.items():
        assert 'rec_username' in spec['definitions']
        assert 'users' in spec['paths'][
            '/v1/decorated/{username}'
        ]['get']['tags']

if __name__ == "__main__":
    app.run(debug=True)



"""
In this example `openapi` version is used instead of `swagger` version.
"""
from flask import Flask
from flasgger import Swagger

app = Flask(__name__)
swag = Swagger(app, config={
    'headers': [],
    'specs': [
        {
            'endpoint': 'apispec',
            'route': '/apispec.json'
        }
    ],
    'openapi': '3.0.1'
})


def test_swag(client, specs_data):
    """
    This test is runs automatically in Travis CI

    :param client: Flask app test client
    :param specs_data: {'url': {swag_specs}} for every spec in app
    """
    for spec in specs_data.values():
        assert 'openapi' in spec
        assert '3.0.1' == spec['openapi']
        assert 'swagger' not in spec


if __name__ == '__main__':
    app.run(debug=True)



"""
A test using simple class based definitions
"""
from flask import Flask, jsonify, request
from flask.views import View

from flasgger import Swagger

app = Flask(__name__)

# Flasgger is initialized like a standard flask extension.
# You can also use .init_app() with the "app factory" pattern.
swag = Swagger(app)


# Definitions can be registered as objects or functions.
# Docstring lines (before '---') are used as the description.
# YAML after '---' defines the Swagger definition schema.
@swag.definition('return_test_1')
class ReturnTest(object):
    """
    test return
    ---
    properties:
      result:
        type: string
        description: The test
        default: 'test1'
    """

    def __init__(self, size):
        self.size = int(size)

    def dump(self):
        return {"result": "test1" * self.size}


@swag.definition('return_test_2')
def return_test(size):
    """
    another test return
    ---
    properties:
      result:
        type: string
        description: The test
        default: 'test2'
    """
    size = int(size)

    return {"result": "test2" * size}


# Flask endpoints with flasgger docstrings are automatically registered.
# The first line of the docstring is used as the summary/
# The following lines (before '---') are used as the description.
# YAML after '---' defines the Swagger path schema.
@app.route("/recs", methods=['GET'])
def recs():
    """
    A simple test API
    This endpoint does nothing
    Only returns "test"
    ---
    tags:
      - testapi
    parameters:
      - name: size
        in: query
        type: string
        description: size of elements
    responses:
      200:
        description: A single user item
        schema:
          $ref: '#/definitions/return_test_1'
    """
    message = ReturnTest(request.args.get('size', 1)).dump()
    return jsonify(message)


class Foo(View):
    methods = ['GET', 'POST']

    def dispatch_request(self):
        """
        A simple test API
        This endpoint does nothing
        Only returns "test"
        ---
        tags:
          - rohit
        parameters:
          - name: size
            in: query
            type: string
            description: size of elements
        responses:
          200:
            description: A single user item
            schema:
              $ref: '#/definitions/return_test_2'
        """
        message = return_test(request.args.get('size', 1))
        return jsonify(message)


app.add_url_rule('/dispatch_request', view_func=Foo.as_view('foo'))

if __name__ == "__main__":
    app.run(debug=True)



"""
Example of swagger document auto loading by endpoint
name(view function/method name), parsed by
flask_restful.reqparse.RequestParser and validated by jsonschema
"""
import os

from flask import Flask, request, jsonify, Response
from flask.views import MethodView
from flasgger import Swagger


app = Flask(__name__)
app.config['SWAGGER'] = {
    'title': 'Flasgger Parsed Method/Function View Example',
    'doc_dir': './examples/docs/'
}
swag = Swagger(
    app,
    template_file=os.path.join(
        os.getcwd(), 'examples', 'docs', 'template.yml'),
    parse=True)


class ItemsView(MethodView):
    """
    Flasgger will try to load "./examples/docs/items/{method}.yml" as
    swagger document
    """
    def get(self):
        """
        If we set "parse" is True in Flasgger app, we will get parsed and
        validated data stored in "flask.request.parsed_data".

        In "parsed_data", different location's var stored in different key,
        there is a map between RequestParser's location and swagger
        doc's "in" parameter, eg: 'query' -> 'args'.See
        "Swagger.SCHEMA_LOCATIONS" for more locations
        """
        return jsonify(
            [{'name': 'test', 'id': 1,
              'type': request.parsed_data['args']['type']},
             {'name': 'test2', 'id': 2,
              'type': request.parsed_data['args']['type']}])

    def post(self):
        return jsonify(
            {'name': request.parsed_data['json']['name'], 'id': 3,
             'type': request.parsed_data['json']['type']})


class ItemMethodView(MethodView):
    def get(self, id):
        return jsonify({'name': 'test', 'id': id, 'type': 'NORMAL'})

    def put(self, id):
        return jsonify(
            {'name': request.parsed_data['json']['name'],
             'id': 3, 'type': 'NORMAL'})


class EmptyView(MethodView):
    """In this view, we do not provide api doc"""
    def get(self):
        return Response(status=200)


@app.route('/api/users/<group>/', methods=['POST'])
def users(group):
    """Create one user with nested json body.
    ---
    tags:
    - FunctionView
    parameters:
    - name: group
      in: path
      type: string
      required: true
    - name: User
      in: body
      schema:
        type: object
        required:
        - data
        properties:
          data:
             type: object
             required:
             - name
             - age
             properties:
                age:
                  type: integer
                name:
                  type: string
          tags:
            type: array
            minItems: 1
            items:
              type: integer
    definitions:
      User:
        type: object
        properties:
          id:
            type: integer
          name:
            type: string
          group:
            type: string
          age:
            type: integer
    responses:
      200:
        description: ok
        schema:
          $ref: '#/definitions/User'
        examples:
          {'id': 1, 'name': 'test', 'group': 1, 'age': 20}
      400:
        description: Miss data
    """
    return jsonify(
        {'id': 1,
         'name': request.parsed_data['json']['data']['name'],
         'age': request.parsed_data['json']['data']['age'],
         'group': request.parsed_data['path']['group']})


@app.route('/api/user/')
def user():
    """Flasgger will try to load './examples/docs/user.yml' as swagger doc
    """
    return jsonify({'id': request.parsed_data['args']['id'], 'name': 'test'})


app.add_url_rule(
    '/api/items/', view_func=ItemsView.as_view(name='items'),
    methods=['GET', 'POST'])
app.add_url_rule(
    '/api/items/<int:id>/', view_func=ItemMethodView.as_view(name='item'),
    methods=['GET', 'PUT'])
app.add_url_rule(
    '/api/empty/', view_func=EmptyView.as_view(name='empty'),
    methods=['GET'])


def test_swag(client, specs_data):
    """
    This test is runs automatically in Travis CI

    :param client: Flask app test client
    :param specs_data: {'url': {swag_specs}} for every spec in app
    """
    res = client.get('/api/items/')
    assert res.status_code == 400
    res = client.get('/api/items/?type=myLengthIsTooLong')
    assert res.status_code == 400
    res = client.get('/api/items/?type=NORMAL')

    assert res.status_code == 200
    res = client.get('/api/items/1/')
    assert res.status_code == 200
    res = client.put('/api/items/1/')
    assert res.status_code == 400
    res = client.put('/api/items/1/', json={'name': 'test'})
    assert res.status_code == 200

    res = client.post('/api/users/1/', json={'name': 'miss data'})
    assert res.status_code == 400
    res = client.post(
        '/api/users/1/',
        json={'data': {'name': 'test', 'age': 20}, 'tags': ['error_tag']})
    assert res.status_code == 400
    res = client.post(
        '/api/users/1/',
        json={'data': {'name': 'test', 'age': 20}, 'tags': [1, 2]})
    assert res.status_code == 200


if __name__ == '__main__':
    app.run(debug=True)



"""
Example using Marshmallow Schema and fields as definitions
"""
# coding: utf-8
from flask import Flask, jsonify, Response, request

from flasgger import Swagger, SwaggerView

swagger_config = {
    "headers": [],
    "openapi": "3.0.2",
    "components": {
        'schemas': {
            'Species': {
                'type': 'object',
                'properties': {
                    'id': {
                        'type': 'integer',
                        'format': 'int64'
                    },
                    'name': {
                        'type': 'string'
                    }
                }
            },
            'Animal': {
                'type': 'object',
                'properties': {
                    'species': {
                        '$ref': '#/components/schemas/Species'
                    },
                    'name': {
                        'type': 'string'
                    }
                }
            }
        }
    },
    "title": "Animal Shelter API",
    "version": '',
    "termsOfService": "",
    "static_url_path": "/characteristics/static",
    "swagger_ui": True,
    "description": "",
}


class Animals(SwaggerView):

    def get(self):
        """
        find shelter animals that fit the criteria

        ---
        tags:
        - Animals
        description: Parameters can be provided in the query to search
        operationId: getAnimals
        parameters:
            - name: species
              in: query
              required: true
              schema:
                type: string
            - name: status
              in: query
              required: true
              schema:
                type: string
                enum:
                  - adopted
                  - pending
                  - adoptable
                  - transferred
                  - sanctuary
                  - medical-hold
        responses:
            '200':
              description: Animals to be returned
              content:
                application/json:
                  schema:
                    type: array
                    items:
                      $ref: '#/components/schemas/Animal'
        """
        species = request.args.get('name')
        status = request.args.get('status')
        return Response(status=200)


app = Flask(__name__)
swag = Swagger(app, config=swagger_config, merge=True)

app.add_url_rule(
    '/animals',
    view_func=Animals.as_view('animals'),
    methods=['GET']
)

if __name__ == "__main__":
    app.run(debug=True)



# coding: utf-8
from flex.core import validate
from flex import load as validate_fully


def test_validate_example_specs(test_data):
    """
    This test is generated by conftest.py
    for each app in examples app and `test_data` param is:

    mod, client, specs_data, opts = test_data

    mod: examples.example_name (the test module)
    client: Flask app test client
    specs_data: {'url': {swag_specs}} for every spec in app
    opts: a dictionary of test metadata defined in the example

    :expectedresults: validate specs dictionary using flex
    """
    mod, client, specs_data, opts = test_data
    skip_full_validation = opts.get('skip_full_validation', False)
    for url, spec in specs_data.items():
        if 'openapi' not in spec and not skip_full_validation:
            # Flex can do a sophisticated and thorough validatation of
            # Swagger 2.0 specs, before it was renamed to OpenAPI.
            validate_fully(spec)
        else:
            # OpenAPI specs are not yet supported by flex, so we should fall
            # back to a fairly simple structural validation.
            validate(spec)


def test_required_attributes(test_data):
    """
    This test is generated by conftest.py
    for each app in examples app and `test_data` param is:

    mod, client, specs_data, opts = test_data

    mod: examples.example_name (the test module)
    client: Flask app test client
    specs_data: {'url': {swag_specs}} for every spec in app
    opts: a dictionary of test metadata defined in the example

    :expectedresults: assert required attributes are present
    """
    mod, client, specs_data, opts = test_data
    for url, spec in specs_data.items():
        assert 'paths' in spec, 'paths is required'
        assert 'info' in spec, 'info is required'


def test_example_swag(test_data):
    """
    This test is generated by conftest.py
    for each app in examples app and `test_data` param is:

    mod, client, specs_data, opts = test_data

    mod: examples.example_name (the test module)
    client: Flask app test client
    specs_data: {'url': {swag_specs}} for every spec in app
    opts: a dictionary of test metadata defined in the example

    :expectedresults: run mod.test_swag with no errors
    """
    mod, client, specs_data, opts = test_data
    if getattr(mod, 'test_swag', None) is not None:
        mod.test_swag(client, specs_data)



"""
Example loading all specs from JSON template
"""
from flask import Flask, jsonify

from flasgger import Swagger

app = Flask(__name__)
app.config['SWAGGER'] = {
    'title': 'Colors API',
    'uiversion': 2
}
swag = Swagger(app, template_file='colors_template.json')


@app.route('/colors/<palette>/')
def colors(palette):
    """
    Example using a dictionary as specification
    This is the description
    You can also set 'summary' and 'description' in
    specs_dict
    ---
    # values here overrides the specs dict
    deprecated: true
    """
    all_colors = {
        'cmyk': ['cian', 'magenta', 'yellow', 'black'],
        'rgb': ['red', 'green', 'blue']
    }
    if palette == 'all':
        result = all_colors
    else:
        result = {palette: all_colors.get(palette)}

    return jsonify(result)

if __name__ == "__main__":
    app.run(debug=True)



import pytest
from flasgger.base import Swagger


def test_init_config(monkeypatch):

    def __init__(self, config=None, merge=False):
        self._init_config(config, merge)

    monkeypatch.setattr(Swagger, "__init__", __init__)

    # # Unspecified config will be initialized to dict()
    t = Swagger(config=None, merge=False)
    assert t.config == Swagger.DEFAULT_CONFIG

    # Empty dict passed to arguments will be overriden with default_config
    empty_dict = dict()
    t = Swagger(config=empty_dict, merge=False)
    assert t.config == Swagger.DEFAULT_CONFIG
    assert t.config is not empty_dict

    # Config will be merged
    d = {"a": 0}
    t = Swagger(config=d, merge=False)
    assert t.config is d

    # Config will be overridden
    t = Swagger(config={"a": 0}, merge=False)
    assert t.config == {"a": 0}

    # Config will be merged
    t = Swagger(config={"a": 0}, merge=True)
    assert t.config.items() > {"a": 0}.items()
    assert all( t.config[k] == v for k, v in Swagger.DEFAULT_CONFIG.items() )

    # Config will be merged
    empty_dict = dict()
    t = Swagger(config=empty_dict, merge=True)
    assert t.config == Swagger.DEFAULT_CONFIG

    # keys in DEFAULT_CONFIG will be overridden
    d = {"specs": [
        {
            "endpoint": "swagger",
            "route": "/characteristics/swagger.json",
            "rule_filter": lambda rule: True,  # all in
            "model_filter": lambda tag: True,  # all in
        }
    ],}
    t = Swagger(config=d, merge=True)
    assert all( t.config[k] == v for k, v in d.items() )
    assert t.config["specs"] == d["specs"]



import jsonschema
from jsonschema import ValidationError
from werkzeug.exceptions import abort
from flask import Flask
try:
    from http import HTTPStatus
except ImportError:
    import httplib as HTTPStatus
try:
    import simplejson as json
except ImportError:
    import json
from flask import jsonify
from flask import Response
from flask import request
from flasgger import Swagger


def validation_error_inform_error(err, data, schema):
    """
    Custom validation error handler which produces 404 Bad Request
    response in case validation fails and returns the error
    """
    abort(Response(
        json.dumps({'error': str(err), 'data': data, 'schema': schema}),
        status=HTTPStatus.BAD_REQUEST))


def validation_error_404(err, data, schema):
    """
    Custom validation error handler which produces 404 Not Found
    response in case validation fails instead of 400 Bad Request
    """
    abort(Response(status=HTTPStatus.NOT_FOUND))


def validation_error_try_to_accept(err, data, schema):
    """
    Custom validation error handler which attempts alternative
    validation
    """
    if not isinstance(err, ValidationError):
        abort(Response(err, status=HTTPStatus.BAD_REQUEST))

    alernative_schema = dict(schema)
    alernative_schema['properties']['running_time'].update({
        'description': "Films's running time",
        'type': 'integer',
        'example': 169
    })

    try:
        jsonschema.validate(data, alernative_schema)
    except ValidationError as err:
        abort(Response(str(err), status=400))


app = Flask(__name__)
swag = Swagger(app, validation_error_handler=validation_error_inform_error)


@app.route('/film', methods=['POST'])
@swag.validate('Film')
def create_film():
    """
    Film creation endpoint
    ---
    tags:
      - film
    summary: Creates a new Film
    consumes:
      - application/json
    produces:
      - application/json
    parameters:
      - in: body
        name: body
        description:
          Film object that needs to be persisted to the database
        required: true
        schema:
          id: Film
          required:
            - title
            - director
            - distributor
            - release_date
            - running_time
          properties:
            title:
              description: Film's title
              type: string
              example: Interstellar
            director:
              description: Films's director
              type: string
              example: Christopher Nolan
            distributor:
              description: Films's distributor
              type: string
              example: Warner Bros. Pictures
            release_date:
              description: Films's release date
              type: string
              example: October 26, 2014
            running_time:
              description: Films's running time
              type: string
              example: 169 minutes
    responses:
      200:
        description: Successful operation
      400:
        description: Invalid input
    """
    return jsonify(request.json), HTTPStatus.OK


@app.route('/not_found/film', methods=['POST'])
@swag.validate('Film', validation_error_handler=validation_error_404)
def create_film_2():
    """
    Film creation endpoint
    ---
    tags:
      - film
    summary: Creates a new Film
    consumes:
      - application/json
    produces:
      - application/json
    parameters:
      - in: body
        name: body
        description:
          Film object that needs to be persisted to the database
        required: true
        schema:
          $ref: '#/definitions/Film'
    responses:
      200:
        description: Successful operation
      400:
        description: Invalid input
    """
    return jsonify(request.json), HTTPStatus.OK


@app.route('/retry/film', methods=['POST'])
@swag.validate('Film', validation_error_handler=validation_error_try_to_accept)
def create_film_3():
    """
    Film creation endpoint
    ---
    tags:
      - film
    summary: Creates a new Film
    consumes:
      - application/json
    produces:
      - application/json
    parameters:
      - in: body
        name: body
        description:
          Film object that needs to be persisted to the database
        required: true
        schema:
          $ref: '#/definitions/Film'
    responses:
      200:
        description: Successful operation
      400:
        description: Invalid input
    """
    return jsonify(request.json), HTTPStatus.OK


def test_swag(client, specs_data):
    """
    This test is runs automatically in Travis CI

    :param client: Flask app test client
    :param specs_data: {'url': {swag_specs}} for every spec in app
    """
    invalid_film = {
        '_id': "594dba7b2879334e411f3dcc",
        'title': "The Last Airbender",
        'director': "M. Night Shyamalan",
        'distributor': "Paramount Pictures",
        'running_time': 103,
        'release_date': "June 30, 2010"
    }

    super_invalid_film = {
        'title': "The Last Airbender",
        'release_date': 2010
    }

    with client.post(
            '/film', data=json.dumps(invalid_film),
            content_type='application/json') as response:
        assert response.status_code == HTTPStatus.BAD_REQUEST

        received = json.loads(response.data.decode('utf-8'))
        assert received.get('error') is not None
        assert received.get('schema') is not None
        assert received.get('data') == invalid_film

    with client.post(
            '/not_found/film', data=json.dumps(invalid_film),
            content_type='application/json') as response:
        assert response.status_code == HTTPStatus.NOT_FOUND

    with client.post(
            '/retry/film', data=json.dumps(invalid_film),
            content_type='application/json') as response:
        assert response.status_code == HTTPStatus.OK

    with client.post(
            '/retry/film', data=json.dumps(super_invalid_film),
            content_type='application/json') as response:
        assert response.status_code == HTTPStatus.BAD_REQUEST

if __name__ == "__main__":
    app.run(debug=True)



"""
A general example
"""
from flask import Flask, jsonify, request
from flask.views import MethodView

from flasgger import Swagger
from flasgger.utils import swag_from


app = Flask(__name__)


# config your API specs
# you can define multiple specs in the case your api has multiple versions
# ommit configs to get the default (all views exposed in /spec url)
# rule_filter is a callable that receives "Rule" object and
#   returns a boolean to filter in only desired views

app.config['SWAGGER'] = {
    "swagger_version": "2.0",
    "title": "Flasgger",
    "headers": [
        ('Access-Control-Allow-Origin', '*'),
        ('Access-Control-Allow-Methods', "GET, POST, PUT, DELETE, OPTIONS"),
        ('Access-Control-Allow-Credentials', "true"),
    ],
    "specs": [
        {
            "version": "0.0.1",
            "title": "Api v1",
            "endpoint": 'v1_spec',
            "description": 'This is the version 1 of our API',
            "route": '/v1/spec',
            # rule_filter is optional
            # it is a callable to filter the views to extract
            "rule_filter": lambda rule: rule.endpoint.startswith(
                'should_be_v1_only'
            ),
            # definition_filter is optional
            # it is a callable to filter the definition models to include
            "definition_filter": lambda definition: (
                'v1_model' in definition.tags)
        },
        {
            "version": "0.0.2",
            "title": "Api v2",
            "description": 'This is the version 2 of our API',
            "endpoint": 'v2_spec',
            "route": '/v2/spec',
            "rule_filter": lambda rule: rule.endpoint.startswith(
                'should_be_v2_only'
            ),
            "definition_filter": lambda definition: (
                'v2_model' in definition.tags)
        }
    ]
}

swag = Swagger(app)  # you can pass config here Swagger(config={})


@app.after_request
def allow_origin(response):
    response.headers['Access-Control-Allow-Origin'] = 'http://example.com'
    response.headers['Access-Control-Allow-Credentials'] = 'true'

    return response


class UserAPI(MethodView):

    def get(self, team_id):
        """
        Get a list of users
        First line is the summary
        All following lines until the hyphens is added to description
        ---
        tags:
          - users
        parameters:
          - name: team_id
            in: path
            description: ID of team (type any number)
            required: true
            type: integer
            default: 42
        responses:
          200:
            description: Returns a list of users
            schema:
                type: array
                items:
                    $ref: '#/definitions/User'
        """
        data = {
            "users": [
                {"name": "Steven Wilson", "team": team_id},
                {"name": "Mikael Akerfeldt", "team": team_id},
                {"name": "Daniel Gildenlow", "team": team_id}
            ]
        }
        return jsonify(data)

    def post(self):
        """
        Create a new user
        First line is the summary
        All following lines until the hyphens is added to description
        ---
        tags:
          - users
        parameters:
          - in: body
            name: body
            schema:
              id: User
              required:
                - team
                - name
              properties:
                team:
                  type: integer
                  description: team for user
                  default: 42
                name:
                  type: string
                  description: name for user
                  default: Russel Allen
        responses:
          201:
            description: New user created
            schema:
                type: array
                items:
                    $ref: '#/definitions/User'
          import: "not_found.yaml"
        """
        return jsonify(
            {"data": request.json, "status": "New user created"}
        ), 201


view = UserAPI.as_view('users')
app.add_url_rule(
    '/v1/users/<int:team_id>',
    view_func=view,
    methods=['GET'],
    endpoint='should_be_v1_only'
)
app.add_url_rule(
    '/v1/users',
    view_func=view,
    methods=['POST'],
    endpoint='should_be_v1_only_post'
)

# LOADING SPECS FROM EXTERNAL FILE


@app.route('/v1/decorated/<username>', endpoint='should_be_v1_only_username')
@swag_from('username_specs.yml')
def fromfile_decorated(username):
    return jsonify({'username': username})

@app.route('/v1/decorated_no_descr/<username>', endpoint='should_be_v1_only_username_no_descr')
@swag_from('username_specs_no_descr.yml')
def fromfile_decorated_no_descr(username):
    return jsonify({'username': username})

@app.route('/v1/decorated_no_sep/<username>', endpoint='should_be_v1_only_username_no_sep')
@swag_from('username_specs_no_sep.yml')
def fromfile_decorated_no_sep(username):
    return jsonify({'username': username})

@app.route('/v1/decorated_bom/<username>', endpoint='should_be_v1_only_username_bom')
@swag_from('username_specs_bom.yml')
def fromfile_decorated_bom(username):
    return jsonify({'username': username})

@app.route('/v1/decorated_utf16/<username>', endpoint='should_be_v1_only_username_utf16')
@swag_from('username_specs_utf16.yml')
def fromfile_decorated_utf16(username):
    return jsonify({'username': username})

@app.route('/v1/decorated_utf32/<username>', endpoint='should_be_v1_only_username_utf32')
@swag_from('username_specs_utf32.yml')
def fromfile_decorated_utf32(username):
    return jsonify({'username': username})


try:
    from pathlib import Path
    @app.route('/v1/decorated_pathlib_path/<username>',
               endpoint='should_be_v1_only_username_pathlib_path')
    @swag_from(Path('username_specs.yml'))
    def fromfile_decorated(username):
        return jsonify({'username': username})
except ImportError:
    pass

# OR


@app.route('/v1/fileindoc/<username>', endpoint='should_be_v1_only_username_1')
def fromfile_indocstring(username):
    """
    file: username_specs.yml
    """
    return jsonify({'username': username})

@app.route('/v1/fileindoc_no_descr/<username>', endpoint='should_be_v1_only_username_no_descr_1')
def fromfile_indocstring_no_descr(username):
    """
    file: username_specs_no_descr.yml
    """
    return jsonify({'username': username})

@app.route('/v1/fileindoc_no_sep/<username>', endpoint='should_be_v1_only_username_no_sep_1')
def fromfile_indocstring_no_sep(username):
    """
    file: username_specs_no_sep.yml
    """
    return jsonify({'username': username})


# DEFINITIONS FROM OBJECTS

@swag.definition('Hack', tags=['v2_model'])
def hack(subitems):
    """
    Hack Object
    ---
    properties:
      hack:
        type: string
        description: it's a hack
      subitems:
        type: array
        items:
          $ref: '#/definitions/SubItem'
    """
    return {
        'hack': "string",
        'subitems': [subitem.dump() for subitem in subitems]
    }


@swag.definition('SubItem', tags=['v2_model'])
class SubItem(object):
    """
    SubItem Object
    ---
    properties:
      bla:
        type: string
        description: Bla
      blu:
        type: integer
        description: Blu
    """
    def __init__(self, bla, blu):
        self.bla = str(bla)
        self.blu = int(blu)

    def dump(self):
        return dict(vars(self).items())


@app.route("/v2/resource", endpoint="should_be_v2_only")
def bla():
    """
    An endpoint that isn't using method view
    ---
    tags:
    - hacking
    responses:
      200:
        description: Hacked some hacks
        schema:
          $ref: '#/definitions/Hack'
    """
    subitems = [SubItem("string", 0)]
    return jsonify(hack(subitems))


@swag.definition('rec_query_context', tags=['v2_model'])
class RecQueryContext(object):
    """
    Recommendation Query Context
    ---
    required:
      - origin
    properties:
      origin:
        type: string
        default: sugestao
      last_event:
        type: object
        schema:
          $ref: '#/definitions/rec_query_context_last_event'
    """
    def __init__(self, origin, last_event=None):
        self.origin = origin
        self.last_event = last_event

    def dump(self):
        data = {'origin': self.origin}
        if self.last_event:
            data.update({'last_event': self.last_event.dump()})
        return data


@swag.definition('rec_query_context_last_event', tags=['v2_model'])
class RecQueryContextLastEvent(object):
    """
    RecQueryContext Last Event Definition
    ---
    properties:
      event:
        type: string
        default: apply
      data:
        type: object
        schema:
          $ref: '#/definitions/rec_query_context_last_event_data'
    """
    def __init__(self, event=None, data=None):
        self.event = event
        self.data = data

    def dump(self):
        data = {}
        if self.event:
            data.update({'event': self.event})
        if self.data:
            data.update({'data': self.data.dump()})
        return data


@swag.definition('rec_query_context_last_event_data', tags=['v2_model'])
class RecQueryContextLastEventData(object):
    """
    RecQueryContextLastEvent Data Object
    ---
    properties:
      candidate_id:
        type: integer
        default: 123456
      opening_id:
        type: integer
        default: 324345435
      company_id:
        type: integer
        default: 324345435
      datetime:
        type: string
        format: date-time
        default: 2014-09-10T11:41:00.12343-03:00
      recruiter_id:
        type: integer
        default: 435345
      context:
        $ref: '#/definitions/rec_query_context'
    """
    def __init__(self, candidate_id=None, opening_id=None, company_id=None,
                 datetime=None, recruiter_id=None):
        self.candidate_id = candidate_id
        self.opening_id = opening_id
        self.company_id = company_id
        self.datetime = datetime
        self.recruiter_id = recruiter_id

    def dump(self):
        data = {}
        for var in ['candidate_id', 'opening_id', 'company_id', 'datetime',
                    'recruiter_id']:
            if var in vars(self):
                data.update({var: vars(self)[var]})
        return data


@app.route("/v2/recommendation/<target_type>/<item_type>", methods=['POST'],
           endpoint="should_be_v2_only_recommendation")
def recommend(target_type, item_type):
    """
    Recommendation
    Get a single item_type as recommendation for the target_type
    ---
    tags:
      - recommendation
    parameters:
      - name: target_type
        in: path
        description: currently only "candidate" is supported
        required: true
        type: string
        default: candidate
      - name: item_type
        in: path
        description: currently only "openings" is supported
        required: true
        type: string
        default: openings
      - in: body
        name: body
        schema:
          id: rec_query
          required:
            - candidate_id
            - context
          properties:
            candidate_id:
              type: integer
              description: Id of the target (candidate / user)
              default: 123456
            exclude:
              type: array
              description: item_ids to exclude from recs
              default: [12345, 123456]
              items:
                  type: integer
            context:
              type: object
              schema:
                $ref: '#/definitions/rec_query_context'
    responses:
      200:
        description: A single recommendation item
        schema:
          id: rec_response
          properties:
            opening_id:
              type: integer
              description: The id of the opening
              default: 123456
      204:
         description: No recommendation found
    """

    data = {
        "opening_id": 12312313434
    }
    return jsonify(data)


@app.route("/")
def hello():
    return """
      <h1> Welcome to Flasgger demo application</h1>
      This app exposes two API versions
      <ul>
         <li><a href="/apidocs/index.html?url=/v1/spec">Api Version 1</a></li>
         <li><a href="/apidocs/index.html?url=/v2/spec">Api Version 2</a></li>
      </ul>
    <p>
      As you can see the APIs is served by the same swagger UI on
      <a href="/apidocs/index.html">Api docs</a>
     </p>
     <p>
      Flasgger <a href="https://github.com/rochacbruno/flasgger">
      https://github.com/rochacbruno/flasgger</a>
    </p>
    """

if __name__ == "__main__":
    app.run(debug=True)



"""
A slight variation of the basic auth example to demonstrate using the
"decorators" parameter in init_app, instead of during Swagger object
creation itself.
This is useful in cases when the application factory pattern is used in your
flask app.
The example is very similar to the basic_auth example, except that the
decorators parameter is passed in init_app.
Doc for the original basic auth example follows.

An example that uses HTTP Basic Auth to protect the /apidocs and
/apispec_1.json endpoints.

In this example the login credentials to view the Swagger docs are as follows:

    Username: guest
    Password: secret

Failing to provide the right credentials results in a 401 Unauthorized
response from all the Flasgger endpoints.
"""
from flask import Flask, request, redirect, Response, jsonify
from functools import wraps

from flasgger import Swagger


def requires_basic_auth(f):
    """Decorator to require HTTP Basic Auth for your endpoint."""

    def check_auth(username, password):
        return username == "guest" and password == "secret"

    def authenticate():
        return Response(
            "Authentication required.", 401,
            {"WWW-Authenticate": "Basic realm='Login Required'"},
        )

    @wraps(f)
    def decorated(*args, **kwargs):
        # NOTE: This example will require Basic Auth only when you run the
        # app directly. For unit tests, we can't block it from getting the
        # Swagger specs so we just allow it to go thru without auth.
        # The following two lines of code wouldn't be needed in a normal
        # production environment.
        if __name__ != "__main__":
            return f(*args, **kwargs)

        auth = request.authorization
        if not auth or not check_auth(auth.username, auth.password):
            return authenticate()
        return f(*args, **kwargs)

    return decorated


app = Flask(__name__)
app.config["SWAGGER"] = {
    "title": "Swagger Basic Auth App",
    "uiversion": 2,
}
swag = Swagger(
    template={
        "swagger": "2.0",
        "info": {
            "title": "Swagger Basic Auth App",
            "version": "1.0",
        },
        "consumes": [
            "application/json",
        ],
        "produces": [
            "application/json",
        ],
    },
)
# passing decorators in init_app
swag.init_app(app, decorators=[requires_basic_auth])


@app.route("/echo/<name>", methods=["GET", "POST"])
def echo(name):
    """
    Echo back the name and any posted parameters.
    ---
    tags:
      - echo
    parameters:
      - in: path
        name: name
        type: string
        required: true
      - in: body
        name: body
        description: JSON parameters.
        schema:
          properties:
            first_name:
              type: string
              description: First name.
              example: Alice
            last_name:
              type: string
              description: Last name.
              example: Smith
            dob:
              type: string
              format: date
              description: Date of birth.
              example: 1990-01-01
            comment:
              type: string
              description: Something arbitrary.
              example: Hello world
    responses:
      200:
        description: OK.
    """
    data = {
        "url_name": name,
        "json": request.json,
    }
    return jsonify(data)


@app.route("/")
def index():
    return redirect("/apidocs")


if __name__ == "__main__":
    app.run(debug=True)



"""
Example using marshmallow APISpec as base template for Flasgger specs
"""
# coding: utf-8
from flask import Flask, jsonify

from flasgger import APISpec, Schema, Swagger, fields
from apispec.ext.marshmallow import MarshmallowPlugin
from apispec_webframeworks.flask import FlaskPlugin

# Create an APISpec
spec = APISpec(
    title='Flasger Petstore',
    version='1.0.10',
    openapi_version='2.0',
    plugins=[
        FlaskPlugin(),
        MarshmallowPlugin(),
    ],
)

app = Flask(__name__)


# Optional marshmallow support
class CategorySchema(Schema):
    id = fields.Int()
    name = fields.Str(required=True)


class PetSchema(Schema):
    category = fields.Nested(CategorySchema, many=True)
    name = fields.Str()


@app.route('/random')
def random_pet():
    """
    A cute furry animal endpoint.
    Get a random pet
    ---
    description: Get a random pet
    responses:
        200:
            description: A pet to be returned
            schema:
                $ref: '#/definitions/Pet'
    """
    pet = {'category': [{'id': 1, 'name': 'rodent'}], 'name': 'Mickey'}
    return jsonify(PetSchema().dump(pet).data)


template = spec.to_flasgger(
    app,
    definitions=[CategorySchema, PetSchema],
    paths=[random_pet]
)

"""
optionally if using apispec.APISpec from original module
you can do:

from flasgger.utils import apispec_to_template
template = apispec_to_template(
    app=app,
    spec=spec,
    definitions=[CategorySchema, PetSchema],
    paths=[random_pet]
)

"""

# start Flasgger using a template from apispec
swag = Swagger(app, template=template)


if __name__ == '__main__':
    app.run(debug=True)



"""0.9.0 News and Changes

- Updated Swagger UI 3.x
- uiversion now defaults to 3
- 'hide_top_bar' config option to remove the green top bar
- 'fotter_text' added to customize the footer text (allows html and <script>)
- templates/flasgger/footer.html added and can be replaced to customization
  this footer.html is the right place for custom <script>
- 'top_text' added to customize the header text (allows html)
- templates/flasgger/top.html added and can be replaced to customization
- 'head_text' added to customize the <head> (allows html)
- templates/flasgger/head.html added and can be replaced to customization
- added 'doc_expansion' config to control the collapse
- added 'ui_params' to allow override of any swagger.ui values
"""

from flask import Flask, request
from flasgger import Swagger

app = Flask(__name__)
app.config['TEMPLATES_AUTO_RELOAD'] = True
app.config['SWAGGER'] = {
    'title': 'MyAPI',

    # UI version 3 is now the default
    # 'uiversion': '3',

    # This setup hides the green top bar
    'hide_top_bar': True,

    # this text is rendered in the footer
    # (optionally you can replace flasgger/footer.html template)
    'footer_text': '<b>Hello World </b><script>alert("Hello World!")</script>',

    # this text is rendered in the header
    # (optionally you can replace flasgger/header.html template)
    'top_text': '<b><span class="top_text">Welcome to my api </span></b>',

    # this text is rendered in the <head>
    # (optionally you can replace flasgger/head.html template)
    'head_text': '<style>.top_text{color: red;}</style>',

    # Control the collapse of each tag, '"none"' means all tags default closed
    # "none" - It'll Hide everything.
    # "list"- It'll expand/List all the operations only. (default)
    # "full" - It'll expand everything(Full expand as the name says).
    'doc_expansion': "list",

    # Allows overriding any of the uiparams
    # This is useful to override other stuff not provided by the above aliases
    'ui_params': {
        'apisSorter': 'alpha',
        'operationsSorter': 'alpha',
    },

    # Allows overriding any of the uiparams with Javascript expressions
    # This is useful to override other stuff not provided by the above aliases which cannot be serialized to a JSON string
    'ui_params_text': '''{
    "operationsSorter" : (a, b) => a.get("path").localeCompare(b.get("path"))
}'''
}

Swagger(app)


@app.route('/', methods=['GET', 'POST'])
def main_route():
    """
    Test Endpoint
    ---
    tags:
        -   Test
    parameters:
        -   name: data
            in: formData
            required: True
            type: string
            description: data to send
    responses:
        200:
            description: data received successfully
        404:
            description: data not found in request form
    """
    if 'data' not in request.form.keys():
        return 'data not found in request form', 404
    return 'data received: ' + str(request.form['data'])


if __name__ == '__main__':
    app.run()



"""
This example demonstrates usage of JWT token authentication with
Flasgger-enabled application.

Swagger version used is 3, although 2 will also run just fine. For more details
on how to use Flask-JWT in general, take a look at the:
https://pythonhosted.org/Flask-JWT.

In this example the login credentials to call the protected API are as follows:

    Username: guest
    Password: secret

In order to access protected API methods, user must first login with valid
credentials. Successful login appends JWT token in login response header
(name of the field is: jwt-token) - this is the field that Swagger-UI will
search for when gets the response and if found - it will be automatically use
its value when issuing next requests. These requests have Authorization HTTP
header field with content set to: JWT token_value_goes_here (note the space
between JWT and token value).

Benefit of this solution is that once token is obtained, there is no need to
enter it manually while working. Consider a good practice to implement method
to refresh token, in order to prevent token expiration.

Note however that if you want to use JWT_AUTH_HEADER_NAME - you must provide
your own JWT request handler. This is because flask-jwt is not actively
maintained anymore, making it impossible to add JWT_AUTH_HEADER_NAME to it. In
normal circumstances, you will not need to use this field, but if you want to
protect your Flasgger page in order to prevent unauthorized access to it by
using basic HTTP auth on some web-server you will have to.
"""


from flask import Flask, jsonify, request
from flask_jwt import JWT, jwt_required, current_identity, JWTError
from werkzeug.security import safe_str_cmp
from flasgger import Swagger


class User(object):
    def __init__(self, user_id, username, password):
        self.id = user_id
        self.username = username
        self.password = password

    def __str__(self):
        return "User(id='%s')" % self.id


users = [
    User(1, 'guest', 'secret'),
]

username_table = {u.username: u for u in users}
userid_table = {u.id: u for u in users}


def authenticate(username, password):
    user = username_table.get(username, None)
    if user and safe_str_cmp(user.password.encode('utf-8'), password.encode('utf-8')):
        return user


def identity(payload):
    user_id = payload['identity']
    return userid_table.get(user_id, None)


app = Flask(__name__)
app.debug = True
app.config["SECRET_KEY"] = "super-secret"
app.config["SWAGGER"] = {
    "title": "Swagger JWT Authentiation App",
    "uiversion": 3,
}
app.config['JWT_AUTH_URL_RULE'] = '/api/auth'
app.config['JWT_AUTH_HEADER_NAME'] = 'JWTAuthorization'

swag = Swagger(app,
    template={
        "openapi": "3.0.0",
        "info": {
            "title": "Swagger Basic Auth App",
            "version": "1.0",
        },
        "consumes": [
            "application/x-www-form-urlencoded",
        ],
        "produces": [
            "application/json",
        ],
    },
)


def jwt_request_handler():
    auth_header_name = app.config['JWT_AUTH_HEADER_NAME']
    auth_header_value = request.headers.get(auth_header_name, None)
    auth_header_prefix = app.config['JWT_AUTH_HEADER_PREFIX']

    if not auth_header_value:
        return

    parts = auth_header_value.split()

    if parts[0].lower() != auth_header_prefix.lower():
        raise JWTError('Invalid JWT header', 'Unsupported authorization type')
    elif len(parts) == 1:
        raise JWTError('Invalid JWT header', 'Token missing')
    elif len(parts) > 2:
        raise JWTError('Invalid JWT header', 'Token contains spaces')

    return parts[1]


jwt = JWT(app, authenticate, identity)
jwt.request_handler(jwt_request_handler)


@app.route("/login", methods=["POST"])
def login():
    """
    User authenticate method.
    ---
    description: Authenticate user with supplied credentials.
    parameters:
      - name: username
        in: formData
        type: string
        required: true
      - name: password
        in: formData
        type: string
        required: true
    responses:
      200:
        description: User successfully logged in.
      400:
        description: User login failed.
    """
    try:
        username = request.form.get("username")
        password = request.form.get("password")

        user = authenticate(username, password)
        if not user:
            raise Exception("User not found!")

        resp = jsonify({"message": "User authenticated"})
        resp.status_code = 200

        access_token = jwt.jwt_encode_callback(user)

        # add token to response headers - so SwaggerUI can use it
        resp.headers.extend({'jwt-token': access_token})

    except Exception as e:
        resp = jsonify({"message": "Bad username and/or password"})
        resp.status_code = 401

    return resp


@app.route("/protected", methods=["GET"])
@jwt_required()
def protected():
    """
    Protected content method.
    ---
    description: Protected content method. Can not be seen without valid token.
    responses:
      200:
        description: User successfully accessed the content.
    """
    resp = jsonify({"protected": "{} - you saw me!".format(current_identity)})
    resp.status_code = 200

    return resp


if __name__ == '__main__':
    app.run()



"""
Example using Marshmallow Schema and fields as definitions
"""
# coding: utf-8
from flask import Flask, jsonify

from flasgger import Schema, Swagger, SwaggerView, fields


class Color(Schema):
    name = fields.Str()


class Palette(Schema):
    pallete_name = fields.Str()
    colors = fields.Nested(Color, many=True)


class PaletteView(SwaggerView):
    parameters = [
        {
            "name": "palette",
            "in": "path",
            "type": "string",
            "enum": ["all", "rgb", "cmyk"],
            "required": True,
            "default": "all"
        }
    ]
    # definitions = {'Palette': Palette, 'Color': Color}
    responses = {
        200: {
            "description": "A list of colors (may be filtered by palette)",
            "schema": Palette
        }
    }

    def get(self, palette):
        """
        Colors API using schema
        This example is using marshmallow schemas
        """
        all_colors = {
            'cmyk': ['cian', 'magenta', 'yellow', 'black'],
            'rgb': ['red', 'green', 'blue']
        }
        if palette == 'all':
            result = all_colors
        else:
            result = {palette: all_colors.get(palette)}
        return jsonify(result)


app = Flask(__name__)
swag = Swagger(app)

app.add_url_rule(
    '/colors/<palette>',
    view_func=PaletteView.as_view('colors'),
    methods=['GET']
)

if __name__ == "__main__":
    app.run(debug=True)



try:
    import simplejson as json
except ImportError:
    import json
try:
    from http import HTTPStatus
except ImportError:
    import httplib as HTTPStatus
from flask import Flask
from flask import jsonify
from flasgger import Swagger
from flasgger import swag_from

app = Flask(__name__)
swag = Swagger(app)


@app.route("/officer/<int:priority>", methods=['POST'])
@swag_from("officer_specs.yml")
def create_officer(priority):
    return 'Request for officer creation successfully received' \
           ' (priority: %i)'.format(priority), HTTPStatus.OK


@app.route('/schema/<string:schema_id>', methods=['GET'])
def get_schema(schema_id):
    """
    Test schema retrieval

    This endpoint returns a schema known to Flasgger
    ---
    tags:
      - schema
    produces:
      - application/json
    parameters:
      - in: path
        name: schema_id
        type: string
        description: schema_id to be retrieved
        required: true

    responses:
      200:
        description: The requested schema
    """
    return jsonify(swag.get_schema(schema_id)), HTTPStatus.OK


def test_swag(client, specs_data):
    """
    This test is runs automatically in Travis CI

    :param client: Flask app test client
    :param specs_data: {'url': {swag_specs}} for every spec in app
    """
    response = client.get('/schema/officer')
    assert response.status_code == HTTPStatus.OK

    retrieved_schema = json.loads(response.data.decode('utf-8'))
    actual_schema = specs_data['/apispec_1.json']['definitions']['Officer']
    try:
        assert retrieved_schema.viewitems() >= actual_schema.viewitems()
    except AttributeError:
        assert retrieved_schema.items() >= actual_schema.items()

if __name__ == "__main__":
    app.run(debug=True)



"""
Example of JSON body validation in POST with various kinds of specs and views.
"""
try:
    from http import HTTPStatus
except ImportError:
    import httplib as HTTPStatus
from flask import Blueprint
from flask import Flask
from flask import jsonify
from flask import request
from flasgger import Schema
from flasgger import Swagger
from flasgger import SwaggerView
from flasgger import fields
from flasgger import swag_from
from flasgger import validate

# Examples include intentionally invalid defaults to demonstrate validation.
_TEST_META_SKIP_FULL_VALIDATION = True

app = Flask(__name__)
swag = Swagger(app)


test_specs_1 = {
  "tags": [
    "users"
  ],
  "parameters": [
    {
      "name": "body",
      "in": "body",
      "required": True,
      "schema": {
        "id": "User",
        "required": [
          "username",
          "age"
        ],
        "properties": {
          "username": {
            "type": "string",
            "description": "The user name.",
            "default": "Sirius Black"
          },
          "age": {
            "type": "integer",
            "description": "The user age (should be integer)",
            "default": "180"
          },
          "tags": {
            "type": "array",
            "description": "optional list of tags",
            "default": [
              "wizard",
              "hogwarts",
              "dead"
            ],
            "items": {
              "type": "string"
            }
          }
        }
      }
    }
  ],
  "responses": {
    "200": {
      "description": "A single user item",
      "schema": {
        "$ref": "#/definitions/User"
      }
    }
  }
}


@app.route("/manualvalidation", methods=['POST'])
@swag_from("test_validation.yml")
def manualvalidation():
    """
    In this example you need to call validate() manually
    passing received data, Definition (schema: id), specs filename
    """
    data = request.json
    validate(data, 'User', "test_validation.yml")
    return jsonify(data)


@app.route("/validateannotation", methods=['POST'])
@swag.validate('User')
@swag_from("test_validation.yml")
def validateannotation():
    """
    In this example you use validate(schema_id) annotation on the
    method in which you want to validate received data
    """
    data = request.json
    return jsonify(data)


@app.route("/autovalidation", methods=['POST'])
@swag_from("test_validation.yml", validation=True)
def autovalidation():
    """
    Example using auto validation from yaml file.
    In this example you don't need to call validate() because
    `validation=True` on @swag_from does that for you.
    In this case it will use the same provided filename
    and will extract the schema from `in: body` definition
    and the data will default to `request.json`

    or you can specify:
    @swag_from('file.yml',
               validation=True,
               definition='User',
               data=lambda: request.json,  # any callable
               )
    """
    data = request.json
    return jsonify(data)


@app.route("/autovalidationfromspecdict", methods=['POST'])
@swag_from(test_specs_1, validation=True)
def autovalidation_from_spec_dict():
    """
    Example using data from dict to validate.
    In this example you don't need to call validate() because
    `validation=True` on @swag_from does that for you.
    In this case it will use the same provided filename
    and will extract the schema from `in: body` definition
    and the data will default to `request.json`

    or you can specify:
    @swag_from('file.yml',
               validation=True,
               definition='User',
               data=lambda: request.json,  # any callable
               )
    """
    data = request.json
    return jsonify(data)


class User(Schema):
    username = fields.Str(required=True, default="Sirius Black")
    # wrong default "180" to force validation error
    age = fields.Int(required=True, min=18, default="180")
    tags = fields.List(fields.Str(), default=["wizard", "hogwarts", "dead"])


class UserPostView(SwaggerView):
    tags = ['users']
    parameters = User
    responses = {
        200: {
            'description': 'A single user item',
            'schema': User
        }
    }
    validation = True

    def post(self):
        """
        Example using marshmallow Schema
        validation=True forces validation of parameters in body
        ---
        # This value overwrites the attributes above
        deprecated: true
        """
        return jsonify(request.json)


app.add_url_rule(
    '/schemevalidation',
    view_func=UserPostView.as_view('schemevalidation'),
    methods=['POST']
)


# ensure the same works for blueprints

example_blueprint = Blueprint(
    "example", __name__, url_prefix='/blueprint')


@example_blueprint.route("/autovalidationfromdocstring", methods=['POST'])
@swag.validate('Officer')
def autovalidation_from_docstring():
    """
    Test validation using JsonSchema

    The default payload is invalid, try it, then change the age to a
    valid integer and try again
    ---
    tags:
      - officer
    parameters:
      - name: body
        in: body
        required: true
        schema:
          id: Officer
          required:
            - name
            - age
          properties:
            name:
              type: string
              description: The officer's name.
              default: "James T. Kirk"
            age:
              type: integer
              description: The officer's age (should be integer)
              default: "138"
            tags:
              type: array
              description: optional list of tags
              default: ["starfleet", "captain", "enterprise", "dead"]
              items:
                type: string

    responses:
      200:
        description: A single officer item
        schema:
          $ref: '#/definitions/Officer'
    """
    data = request.json
    return jsonify(data)


@example_blueprint.route('/manualvalidation', methods=['POST'])
@swag_from("test_validation.yml")
def manualvalidation_bp():
    """
    In this example you need to call validate() manually
    passing received data, Definition (schema: id), specs filename
    """
    data = request.json
    validate(data, 'User', "test_validation.yml")
    return jsonify(data)


@example_blueprint.route('/autovalidation', methods=['POST'])
@swag_from("test_validation.yml", validation=True)
def autovalidation_bp():
    """
    Example using auto validation from yaml file.
    In this example you don't need to call validate() because
    `validation=True` on @swag_from does that for you.
    In this case it will use the same provided filename
    and will extract the schema from `in: body` definition
    and the data will default to `request.json`

    or you can specify:
    @swag_from('file.yml',
               validation=True,
               definition='User',
               data=lambda: request.json,  # any callable
               )
    """
    data = request.json
    return jsonify(data)


@example_blueprint.route("/autovalidationfromspecdict", methods=['POST'])
@swag_from(test_specs_1, validation=True)
def autovalidation_from_spec_dict_bp():
    """
    Example using data from dict to validate.
    In this example you don't need to call validate() because
    `validation=True` on @swag_from does that for you.
    In this case it will use the same provided filename
    and will extract the schema from `in: body` definition
    and the data will default to `request.json`

    or you can specify:
    @swag_from('file.yml',
               validation=True,
               definition='User',
               data=lambda: request.json,  # any callable
               )
    """
    data = request.json
    return jsonify(data)


class BPUserPostView(SwaggerView):
    tags = ['users']
    parameters = User
    responses = {
        200: {
            'description': 'A single user item',
            'schema': User
        }
    }
    validation = True

    def post(self):
        """
        Example using marshmallow Schema
        validation=True forces validation of parameters in body
        ---
        # This value overwrites the attributes above
        deprecated: true
        """
        return jsonify(request.json)


example_blueprint.add_url_rule(
    '/schemevalidation',
    view_func=BPUserPostView.as_view('schemevalidation'),
    methods=['POST']
)


app.register_blueprint(example_blueprint)


def test_swag(client, specs_data):
    """
    This test is runs automatically in Travis CI

    :param client: Flask app test client
    :param specs_data: {'url': {swag_specs}} for every spec in app
    """

    apispec = specs_data.get('/apispec_1.json')

    assert apispec is not None

    paths = apispec.get('paths')

    expected_user_paths = (
        '/autovalidation',
        '/validateannotation',
        '/autovalidationfromspecdict',
        '/blueprint/autovalidation',
        '/blueprint/autovalidationfromspecdict',
        '/blueprint/manualvalidation',
        '/blueprint/schemevalidation',
        '/manualvalidation',
        '/schemevalidation',
    )

    expected_officer_paths = (
        '/blueprint/autovalidationfromdocstring',
    )

    invalid_users = (
        """
        {
          "username": "Sirius Black",
          "age": "180",
          "tags": [
            "wizard",
            "hogwarts",
            "dead"
          ]
        }
        """,
        """
        {
          "age": 180,
          "tags": [
            "wizard"
          ]
        }
        """,
    )

    valid_users = (
        """
        {
          "username": "Sirius Black",
          "age": 180,
          "tags": [
            "wizard",
            "hogwarts",
            "dead"
          ]
        }
        """,
        """
        {
          "username": "Ronald Weasley",
          "age": 22
        }
        """,
    )

    invalid_officers = (
        """
        {
          "name": "James T. Kirk",
          "age": "138",
          "tags": [
            "captain",
            "enterprise",
            "dead"
          ]
        }
        """,
        """
        {
          "age": 138,
          "tags": [
            "captain"
          ]
        }
        """,
    )

    valid_officers = (
        """
        {
          "name": "James T. Kirk",
          "age": 138,
          "tags": [
            "captain",
            "enterprise",
            "dead"
          ]
        }
        """,
        """
        {
          "name": "Jean-Luc Picard",
          "age": 60
        }
        """,
    )

    assert paths is not None and len(paths) > 0

    definitions = apispec.get('definitions')

    assert definitions is not None
    assert definitions.get('User') is not None
    assert definitions.get('Officer') is not None

    for expected_path in expected_user_paths:
        assert paths.get(expected_path) is not None

        for invalid_user in invalid_users:
            response = client.post(
                expected_path, data=invalid_user,
                content_type='application/json')
            assert response.status_code == HTTPStatus.BAD_REQUEST

        for valid_user in valid_users:
            response = client.post(
                expected_path, data=valid_user,
                content_type='application/json')
            assert response.status_code == HTTPStatus.OK

    for expected_path in expected_officer_paths:
        assert paths.get(expected_path) is not None

        for invalid_officer in invalid_officers:
            response = client.post(
                expected_path, data=invalid_officer,
                content_type='application/json')
            assert response.status_code == HTTPStatus.BAD_REQUEST

        for valid_officer in valid_officers:
            response = client.post(
                expected_path, data=valid_officer,
                content_type='application/json')
            assert response.status_code == HTTPStatus.OK

if __name__ == "__main__":
    app.run(debug=True)



"""
Another example using Marshmallow schemas in SwaggerView (MethodView)
"""
# coding: utf-8

from flask import Flask, jsonify, request

from flasgger import Schema, Swagger, SwaggerView, fields

app = Flask(__name__)
app.config['SWAGGER'] = {
    "title": "API using Marshmallow",
    "uiversion": 2
}

swag = Swagger(app)


class User(Schema):
    username = fields.Str(required=True)
    age = fields.Int(required=True, min=18)
    tags = fields.List(fields.Str())


class UserPostView(SwaggerView):

    parameters = User

    # parameters = [
    #     {
    #         'name': 'body',
    #         'in': 'body',
    #         'schema': User
    #     }
    # ]

    responses = {
        200: {
            'description': 'A single user',
            'schema': User
        }
    }

    # optional
    # definitions = {
    #     'User': User  # if nto informed will be extracted from usage above
    # }
    tags = ['users']
    # consumes = ['application/json']
    # produces = ['application/json']
    # schemes = ['http', 'https']
    # security = []
    # deprecated = False
    # operationId = 'user'
    # externalDocs = {'description': 'foo', 'url': 'bar.com'}
    summary = "Will be overwritten by first line of docstring"
    description = "will be overwritten by otehr lines"

    def post(self):
        """
        A simple post
        Do it
        ---
        # This value overwrites the attributes above
        deprecated: true
        """
        return jsonify(request.json)


app.add_url_rule(
    '/user',
    view_func=UserPostView.as_view('user'),
    methods=['POST']
)

if __name__ == "__main__":
    app.run(debug=True)



"""
This tests the use of `decorator` package with Flasgger
"""
from decorator import decorator
from flask import Flask, jsonify
from flasgger import Swagger

app = Flask(__name__)
swag = Swagger(app)


@decorator
def trace(f, *args, **kw):
    kwstr = ', '.join('%r: %r' % (k, kw[k]) for k in sorted(kw))
    print("calling %s with args %s, {%s}" % (f.__name__, args, kwstr))
    return f(*args, **kw)


@app.route('/')
@trace
def index():
    """
    This example tests decorator package
    Should not break in Python 2.7+
    ---
    responses:
      200:
        description: Yeah it works
    """
    return jsonify({'data': 'It works'})


if __name__ == '__main__':
    app.run(debug=True)




# Fix for older setuptools
import re
import os

from setuptools import setup, find_packages


def fpath(name):
    return os.path.join(os.path.dirname(__file__), name)


def read(fname):
    return open(fpath(fname)).read()


def desc():
    return read('README.md')


# grep flasgger/__init__.py since python 3.x cannot
# import it before using 2to3
file_text = read(fpath('flasgger/__init__.py'))


def grep(attrname):
    pattern = r"{0}\W*=\W*'([^']+)'".format(attrname)
    strval, = re.findall(pattern, file_text)
    return strval


setup(
    name='flasgger',
    version=grep('__version__'),
    url='https://github.com/flasgger/flasgger/',
    license='MIT',
    author=grep('__author__'),
    author_email=grep('__email__'),
    description='Extract swagger specs from your flask project',
    long_description=desc(),
    long_description_content_type="text/markdown",
    packages=find_packages(
        exclude=[
            'tests', 'tests.*',
            'examples', 'examples.*',
            'demo_app', 'demo_app.*',
            'etc', 'etc.*'
        ]
    ),
    include_package_data=True,
    zip_safe=False,
    platforms='any',
    install_requires=[
        'Flask>=0.10',
        'PyYAML>=3.0',
        'jsonschema>=3.0.1',
        'mistune',
        'six>=1.10.0'
    ],
    classifiers=[
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
    ]
)



import time
import jsonschema
try:
    import simplejson as json
except ImportError:
    import json
try:
    from http import HTTPStatus
except ImportError:
    import httplib as HTTPStatus
from flask import Flask
from flask import jsonify
from flask import request
from flasgger import Swagger


def drop_id_validate(data, schema):
    """
    Custom validation function which drops parameter '_id' if present
    in data
    """
    jsonschema.validate(data, schema)
    if data.get('_id') is not None:
        del data['_id']


def timestamping_validate(data, schema):
    """
    Custom validation function which inserts a timestamp for when the
    validation occurred
    """
    jsonschema.validate(data, schema)
    data['timestamp'] = str(time.time())


def special_validate(data, schema):
    """
    Custom validation function which inserts an special flag depending
    on the cat's name
    """
    jsonschema.validate(data, schema)
    data['special'] = str(data['name'] == 'Garfield').lower()


def regular_validate(data, schema):
    """
    Regular validation function
    """
    jsonschema.validate(data, schema)


app = Flask(__name__)
swag = Swagger(app, validation_function=drop_id_validate)


@app.route('/cat', methods=['POST'])
@swag.validate('Cat')
def create_cat():
    """
    Cat creation endpoint which drops '_id' parameters when present in
    request data
    ---
    tags:
      - cat
    summary: Creates a new cat
    consumes:
      - application/json
    produces:
      - application/json
    parameters:
      - in: body
        name: body
        description:
          Cat object that needs to be persisted to the database
        required: true
        schema:
          id: Cat
          required:
            - name
            - address
          properties:
            name:
              description: Cat's name
              type: string
              example: Sylvester
            address:
              description: Cat's house address
              type: string
              example: 4000 Warner Blvd., Burbank, CA 91522
    responses:
      200:
        description: Successful operation
      400:
        description: Invalid input
    """
    return jsonify(request.json), HTTPStatus.OK


@app.route('/timestamped/cat', methods=['POST'])
@swag.validate('Cat', validation_function=timestamping_validate)
def create_timestamped_cat():
    """
    Cat creation endpoint which timestamps validated data
    ---
    tags:
      - cat
    summary: Creates a new cat
    consumes:
      - application/json
    produces:
      - application/json
    parameters:
      - in: body
        name: body
        description:
          Cat object that needs to be persisted to the database
        required: true
        schema:
          $ref: '#/definitions/Cat'
    responses:
      200:
        description: Successful operation
        schema:
          $ref: '#/definitions/Cat'
      400:
        description: Invalid input
    """
    return jsonify(request.json), HTTPStatus.OK


@app.route('/special/cat', methods=['POST'])
@swag.validate('Cat', validation_function=special_validate)
def create_special_cat():
    """
    Cat creation endpoint which timestamps validated data
    ---
    tags:
      - cat
    summary: Creates a new cat
    consumes:
      - application/json
    produces:
      - application/json
    parameters:
      - in: body
        name: body
        description:
          Cat object that needs to be persisted to the database
        required: true
        schema:
          $ref: '#/definitions/Cat'
    responses:
      200:
        description: Successful operation
        schema:
          $ref: '#/definitions/Cat'
      400:
        description: Invalid input
    """
    return jsonify(request.json), HTTPStatus.OK


@app.route('/regular/cat', methods=['POST'])
@swag.validate('Cat', validation_function=regular_validate)
def create_regular_cat():
    """
    Cat creation endpoint
    ---
    tags:
      - cat
    summary: Creates a new cat
    consumes:
      - application/json
    produces:
      - application/json
    parameters:
      - in: body
        name: body
        description:
          Cat object that needs to be persisted to the database
        required: true
        schema:
          $ref: '#/definitions/Cat'
    responses:
      200:
        description: Successful operation
        schema:
          $ref: '#/definitions/Cat'
      400:
        description: Invalid input
    """
    return jsonify(request.json), HTTPStatus.OK


def test_swag(client, specs_data):
    """
    This test is runs automatically in Travis CI

    :param client: Flask app test client
    :param specs_data: {'url': {swag_specs}} for every spec in app
    """
    cat = {
        '_id': "594dba7b2879334e411f3dcc",
        'name': "Tom",
        'address': "MGM, 245 N. Beverly Drive, Beverly Hills, CA 90210"
    }

    with client.post(
            '/cat', data=json.dumps(cat), content_type='application/json') as response:
        assert response.status_code == HTTPStatus.OK

        received = json.loads(response.data.decode('utf-8'))
        assert received.get('_id') is None
        assert received.get('timestamp') is None
        assert received.get('special') is None
        try:
            assert received.viewitems() < cat.viewitems()
        except AttributeError:
            assert received.items() < cat.items()

    with client.post(
            '/timestamped/cat', data=json.dumps(cat),
            content_type='application/json') as response:
        assert response.status_code == HTTPStatus.OK

        received = json.loads(response.data.decode('utf-8'))
        assert received.get('_id') == cat.get('_id')
        assert received.get('timestamp') is not None
        assert received.get('special') is None
        try:
            assert received.viewitems() > cat.viewitems()
        except AttributeError:
            assert received.items() > cat.items()

    with client.post(
            '/special/cat', data=json.dumps(cat),
            content_type='application/json') as response:
        assert response.status_code == HTTPStatus.OK

        received = json.loads(response.data.decode('utf-8'))
        assert received.get('_id') == cat.get('_id')
        assert received.get('timestamp') is None
        assert received.get('special') is not None
        try:
            assert received.viewitems() > cat.viewitems()
        except AttributeError:
            assert received.items() > cat.items()

    with client.post(
            '/regular/cat', data=json.dumps(cat),
            content_type='application/json') as response:
        assert response.status_code == HTTPStatus.OK

        received = json.loads(response.data.decode('utf-8'))
        assert received.get('_id') == cat.get('_id')
        assert received.get('timestamp') is None
        assert received.get('special') is None
        try:
            assert received.viewitems() == cat.viewitems()
        except AttributeError:
            assert received.items() == cat.items()

if __name__ == "__main__":
    app.run(debug=True)



"""
A test to ensure that MethodView inheritance works as expected
"""
from flask import Flask, jsonify
from flask.views import MethodView

from flasgger import Swagger


class BaseAPIView(MethodView):
    """BAse view"""


class ModelAPIView(BaseAPIView):
    """Model api view"""


class PostAPIView(ModelAPIView):

    def get(self, team_id):
        """
        Get a list of users
        First line is the summary
        All following lines until the hyphens is added to description
        ---
        tags:
          - users
        parameters:
          - name: team_id
            in: path
            description: ID of team (type any number)
            required: true
            type: integer
        definitions:
          User:
            type: object
            properties:
              name:
                type: string
              team:
                type: integer
        responses:
          200:
            description: Returns a list of users
            schema:
              id: Users
              type: object
              properties:
                users:
                  type: array
                  items:
                    $ref: '#/definitions/User'
            examples:
              users: [{'name': 'Russel Allen', 'team': 66}]
        """
        data = {
            "users": [
                {"name": "Steven Wilson", "team": team_id},
                {"name": "Mikael Akerfeldt", "team": team_id},
                {"name": "Daniel Gildenlow", "team": team_id}
            ]
        }
        return jsonify(data)


app = Flask(__name__)
swag = Swagger(app)

app.add_url_rule(
    '/user/<team_id>',
    view_func=PostAPIView.as_view('user'),
    methods=['GET']
)

if __name__ == "__main__":
    app.run(debug=True)



"""
The simple example using OpenAPI 3.0 callbacks
"""

from flask import Flask, jsonify

from flasgger import Swagger

app = Flask(__name__)
app.config['SWAGGER'] = {
    'title': 'OA3 Callbacks',
    'openapi': '3.0.2'
}
Swagger(app)


@app.route('/run_callback/',  methods=['POST'])
def run_callback():
    """Example endpoint that specifies OA3 callbacks
    This is using docstring for specifications
    ---
    tags:
      - callbacks
    requestBody:
      description: Test
      required: true
      content:
        application/json:
          schema:
            properties:
              callback_url:
                type: string
                format: uri
                description: Callback URL for request
    callbacks:
        onSomeEvent:
          '{$request.body.callback_url}':
            post:
              requestBody:
                description: status payload
                content:
                  application/json:
                    schema:
                      properties:
                        status:
                          type: string
    """

    return jsonify({'result': 'ok'})


def test_swag(client, specs_data):
    """
    This test is runs automatically in Travis CI

    :param client: Flask app test client
    :param specs_data: {'url': {swag_specs}} for every spec in app
    """
    for url, spec in specs_data.items():
        assert 'openapi' in spec
        assert spec['openapi'] == '3.0.2'

        assert 'callbacks' in spec['paths']['/run_callback/']['post']
        assert 'onSomeEvent' in \
               spec['paths']['/run_callback/']['post']['callbacks']


if __name__ == "__main__":
    app.run(debug=True)



"""
Example of Flask RESTFul integration.
requires: `pip install flask-restful`
"""
from flask import Flask
from flask_restful import Api, Resource, abort, reqparse

from flasgger import Swagger, swag_from

app = Flask(__name__)
api = Api(app)
app.config['SWAGGER'] = {
    'title': 'Flasgger RESTful',
    'uiversion': 2
}
swag = Swagger(app)


TODOS = {
    'todo1': {'task': 'build an API'},
    'todo2': {'task': '?????'},
    'todo3': {'task': 'profit!'},
    '42': {'task': 'Use Flasgger'}
}


def abort_if_todo_doesnt_exist(todo_id):
    if todo_id not in TODOS:
        abort(404, message="Todo {} doesn't exist".format(todo_id))


parser = reqparse.RequestParser()
parser.add_argument('task')


# Todo
# shows a single todo item and lets you delete a todo item
class Todo(Resource):
    def get(self, todo_id):
        """
        This is an example
        ---
        tags:
          - restful
        parameters:
          - in: path
            name: todo_id
            required: true
            description: The ID of the task, try 42!
            type: string
        responses:
          200:
            description: The task data
            schema:
              id: Task
              properties:
                task:
                  type: string
                  default: My Task
        """
        abort_if_todo_doesnt_exist(todo_id)
        return TODOS[todo_id]

    def delete(self, todo_id):
        """
        This is an example
        ---
        tags:
          - restful
        parameters:
          - in: path
            name: todo_id
            required: true
            description: The ID of the task, try 42!
            type: string
        responses:
          204:
            description: Task deleted
        """
        abort_if_todo_doesnt_exist(todo_id)
        del TODOS[todo_id]
        return '', 204

    def put(self, todo_id):
        """
        This is an example
        ---
        tags:
          - restful
        parameters:
          - in: body
            name: body
            schema:
              $ref: '#/definitions/Task'
          - in: path
            name: todo_id
            required: true
            description: The ID of the task, try 42!
            type: string
        responses:
          201:
            description: The task has been updated
            schema:
              $ref: '#/definitions/Task'
        """
        args = parser.parse_args()
        task = {'task': args['task']}
        TODOS[todo_id] = task
        return task, 201


# TodoList
# shows a list of all todos, and lets you POST to add new tasks
class TodoList(Resource):
    def get(self):
        """
        This is an example
        ---
        tags:
          - restful
        responses:
          200:
            description: The task data
            schema:
              id: Tasks
              properties:
                task_id:
                  type: object
                  schema:
                    $ref: '#/definitions/Task'
        """
        return TODOS

    def post(self):
        """
        This is an example
        ---
        tags:
          - restful
        parameters:
          - in: body
            name: body
            schema:
              $ref: '#/definitions/Task'
        responses:
          201:
            description: The task has been created
            schema:
              $ref: '#/definitions/Task'
        """
        args = parser.parse_args()
        todo_id = int(max(TODOS.keys()).lstrip('todo')) + 1
        todo_id = 'todo%i' % todo_id
        TODOS[todo_id] = {'task': args['task']}
        return TODOS[todo_id], 201


class Username(Resource):
    @swag_from('username_specs.yml', methods=['GET'])
    def get(self, username):
        return {'username': username}, 200


api.add_resource(TodoList, '/todos')
api.add_resource(Todo, '/todos/<todo_id>')
api.add_resource(Username, '/username/<username>')


if __name__ == '__main__':
    app.run(debug=True)



from flask import Flask
from flask import jsonify
from flasgger import Swagger
from flasgger import swag_from

app = Flask(__name__)
swag = Swagger(app)


@app.route("/example")
@swag_from({
    "responses": {
        400: {
            "description": "Invalid action"
        },
        401: {
            "description": "Login required"
        }
    }
})
def view():
    """
    A test view

    ---
    responses:
      200:
        description: OK
      400:
        description: Modified description
    """
    return jsonify(hello="world")


def test_swag(client, specs_data):
    example_spec = specs_data["/apispec_1.json"]["paths"]["/example"]["get"]
    assert "400" in example_spec["responses"]
    assert "401" in example_spec["responses"]
    assert "200" in example_spec["responses"]

    assert example_spec["responses"]["400"]["description"] == "Modified description"


if __name__ == "__main__":
    app.run(debug=True)



"""
In this example a custom specs_route is set.
"""
from flask import Flask
try:
    from http import HTTPStatus
except ImportError:
    import httplib as HTTPStatus
from flasgger import Swagger

swagger_config = {
    "headers": [
    ],
    "specs": [
        {
            "endpoint": 'specifications',
            "route": '/specifications.json',
            "rule_filter": lambda rule: True,  # all in
            "model_filter": lambda tag: True,  # all in
        }
    ],
    "static_url_path": "/flasgger_static",
    # "static_folder": "static",  # must be set by user
    "specs_route": "/documentation/swagger/"
}

app = Flask(__name__)
swag = Swagger(app, config=swagger_config)


def test_swag(client, specs_data):
    """
    This test is runs automatically in Travis CI

    :param client: Flask app test client
    :param specs_data: {'url': {swag_specs}} for every spec in app
    """
    assert client.get('/documentation/swagger/').status_code == HTTPStatus.OK
    assert specs_data.get('/specifications.json') is not None


if __name__ == '__main__':
    app.run(debug=True)
